#!/usr/share/ucs-test/runner /usr/bin/pytest-3 -l -v

# SPDX-FileCopyrightText: 2021 - 2023 Univention GmbH
#
# SPDX-License-Identifier: AGPL-3.0-only

## -*- coding: utf-8 -*-
## desc: e2e tests for id-broker sddb-builder
## tags: [id-broker]
## exposure: dangerous

import requests
from requests.auth import HTTPBasicAuth
import random

from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple  # noqa: F401
from urllib.parse import urljoin

import pytest
from ucsschool.lib.models.utils import ucr
import univention.testing.strings as uts
import univention.testing.udm as udm_test


APP_ID = "id-broker-sddb-builder"
URL_BASE_PATH = f"/ucsschool/{APP_ID}/v1/"
_localhost_root_url = "https://{}.{}{}".format(ucr["hostname"], ucr["domainname"], URL_BASE_PATH)
API_ROOT_URL = ucr.get("tests/ucsschool/http-api/root_url", _localhost_root_url).rstrip("/") + "/"

API_USERS_GROUP_NAME = f"{APP_ID}-admins"
API_USERS_GROUP_DN = f"cn={API_USERS_GROUP_NAME},cn=groups,{ucr['ldap/base']}"


@pytest.fixture(scope="session")
def resource_urls():
    return {
        "append": urljoin(API_ROOT_URL, "append/"),
        "extend": urljoin(API_ROOT_URL, "extend/"),
    }


@pytest.fixture
def random_append_object():
    def _func():
        return {
            "pseudonym": uts.random_username(),
            "service_provider": "univention-test-app",
            "entry_type": random.choice(["user", "group", "sp_mapping"])
        }
    return _func


def test_enqueue_happy_path_auth(resource_urls, random_append_object):
    username = uts.random_username()
    password = uts.random_username()
    basic_auth = HTTPBasicAuth(username=username, password=password)
    with udm_test.UCSTestUDM() as udm:
        udm.create_user(username=username, groups=API_USERS_GROUP_DN, password=password)
        response = requests.post(url=resource_urls["append"] + "regular", json=random_append_object(), auth=basic_auth)
    assert response.status_code == 201, "response.status_code = {} for URL {!r} -> {!r}".format(
        response.status_code, response.url, response.text
    )


def test_enqueue_unhappy_path_auth(resource_urls, random_append_object):
    attrs = random_append_object()
    username = uts.random_username()
    password = uts.random_username()
    with udm_test.UCSTestUDM() as udm:
        udm.create_user(username=username, groups=API_USERS_GROUP_DN, password=password)
        basic_auth = HTTPBasicAuth(username=username, password="wrong")
        response = requests.post(url=resource_urls["append"] + "regular", json=attrs, auth=basic_auth)
        assert response.status_code == 401, "response.status_code = {} for URL {!r} -> {!r}".format(
            response.status_code, response.url, response.text
        )
        basic_auth = HTTPBasicAuth(username="wrong", password=password)
        response = requests.post(url=resource_urls["append"] + "regular", json=attrs, auth=basic_auth)
        assert response.status_code == 401, "response.status_code = {} for URL {!r} -> {!r}".format(
            response.status_code, response.url, response.text
        )


